@@ -155,7 +155,7 @@ panel_properties_dialog_setup_name_entry (PanelPropertiesDialog *dialog,
 	is_attached = panel_toplevel_get_is_attached (dialog->toplevel);
 
 	name = get_name (dialog->toplevel, is_attached);
-	gtk_entry_set_text (GTK_ENTRY (dialog->name_entry), name);
+	gtk_entry_set_text (GTK_ENTRY (dialog->name_entry), name ? name : "");
 	g_free (name);
 
 	g_signal_connect_swapped (dialog->name_entry, "changed",
@@ -297,25 +297,25 @@ panel_properties_dialog_icon_changed (PanelPropertiesDialog *dialog,
 {
 	const char *icon = NULL;
         char       *freeme = NULL;
-                                                                                                                                                              
+
         icon = gnome_icon_entry_get_filename (entry);
-                                                                                                                                                              
+
         /* Strip dir from the icon path if in the icon
          * theme directory.  See bug #119209
          */
         if (icon && g_path_is_absolute (icon)) {
                 char *dir;
-                                                                                                                                                              
+
                 dir = g_path_get_dirname (icon);
-                                                                                                                                                              
+
                 if (dir && dialog->icon_theme_dir && strcmp (dir, dialog->icon_theme_dir) == 0)
                         icon = freeme = g_path_get_basename (icon);
-                                                                                                                                                              
+
                 g_free (dir);
         }
-                                                                                                                                                              
+
         panel_profile_set_attached_custom_icon (dialog->toplevel, icon);
-                                                                                                                                                              
+
         g_free (freeme);
 }
 
@@ -492,6 +492,11 @@ panel_properties_dialog_opacity_changed (PanelPropertiesDialog *dialog)
 
 	percentage = gtk_range_get_value (GTK_RANGE (dialog->opacity_scale));
 
+	if (percentage >= 98)
+		percentage = 100;
+	else if (percentage <= 2)
+		percentage = 0;
+
 	opacity = (percentage / 100) * 65535;
 
 	panel_profile_set_background_opacity (dialog->toplevel, opacity);
@@ -631,7 +636,7 @@ panel_properties_dialog_response (PanelPropertiesDialog *dialog,
 {
 	switch (response) {
 	case GTK_RESPONSE_CLOSE:
-		gtk_widget_hide (properties_dialog);
+		gtk_widget_destroy (properties_dialog);
 		break;
 	case GTK_RESPONSE_HELP:
 		panel_show_help (gtk_window_get_screen (GTK_WINDOW (properties_dialog)),
@@ -642,6 +647,15 @@ panel_properties_dialog_response (PanelPropertiesDialog *dialog,
 	}
 }
 
+static void
+panel_properties_dialog_destroy (PanelPropertiesDialog *dialog)
+{
+	panel_toplevel_unblock_auto_hide (PANEL_TOPLEVEL (dialog->toplevel));
+	g_object_set_qdata (G_OBJECT (dialog->toplevel),
+			    panel_properties_dialog_quark,
+			    NULL);
+}
+
 static void
 panel_properties_dialog_update_name (PanelPropertiesDialog *dialog,
 				     GConfValue            *value)
@@ -919,6 +933,8 @@ panel_properties_dialog_new (PanelToplevel *toplevel,
 	dialog->properties_dialog = glade_xml_get_widget (gui, "panel_properties_dialog");
 	g_signal_connect_swapped (dialog->properties_dialog, "response",
 				  G_CALLBACK (panel_properties_dialog_response), dialog);
+	g_signal_connect_swapped (dialog->properties_dialog, "destroy",
+				  G_CALLBACK (panel_properties_dialog_destroy), dialog);
 
 	gtk_window_set_screen (GTK_WINDOW (dialog->properties_dialog),
 			       gtk_window_get_screen (GTK_WINDOW (toplevel)));
@@ -966,6 +982,8 @@ panel_properties_dialog_new (PanelToplevel *toplevel,
 	panel_properties_dialog_update_for_attached (dialog,
 						     panel_toplevel_get_is_attached (dialog->toplevel));
 
+	panel_toplevel_block_auto_hide (dialog->toplevel);
+
 	gtk_widget_show (dialog->properties_dialog);
 
 	return dialog;
